import java.awt.*;
import java.awt.geom.*;

/**
 * The Boulder class represents an Asteroids-like boulder that has a 
 * trajectory as well as a current position.  It wraps around when it 
 * gets to the edge of the screen -- or, at least, it will someday if 
 * it doesn't yet.  It's based on the book's Circle class, but with 
 * some extra fields and methods.
 * 
 * @author Brad Richards
 */

public class Boulder
{
    // These fields were all there in the Circle class

    private int diameter;
    private double xPosition;  // This used to be an integer, but is now a double
    private double yPosition;  // This used to be an integer, but is now a double
    private String color;
    private boolean isVisible;

    // These new fields keep track of how quickly the boulder is moving

    private double xVelocity;  	// How far it moves horizontally between steps
    private double yVelocity;  	// How far it moves vertically between steps

    /**
     * Create a new boulder at specified position with specified velocity and size.
     * 
     * @param x     Starting x position
     * @param y     Starting y position
     * @param dx    How many pixels it moves horizontally per update step
     * @param dy    How many pixels it moves vertically per update step
     * @param size  The diameter of the boulder, in pixels
     */
    public Boulder(double x, double y, double dx, double dy, int size)
    {
        // Fill this in.  Your code should store the values passed in
        // through the parameters (e.g. x, y, etc) into the fields at
        // the top of the class so we have access to these values after
        // the constructor has been called. You'll need an assignment
        // statement for each parameter. The instructions on the lab
        // webpage also ask that you set the color to black.
    }

    /**
     * The printStatus method prints information about the canvas (graphics
     * screen) this boulder's on. (It calls accessor methods to get the width 
     * and hight of the canvas object.) Eventually you'll modify this to 
     * report info on the boulder's state as well.
     */
    public void printStatus()
    {
        System.out.println("Boulder is on a canvas that's "+Canvas.getWidth()+" by "+Canvas.getHeight());
    }

    /**
     * Each time this method is called, it calculates a new position for the
     * boulder using the x and y velocities. (It adds the xVelocity to the
     * current x position to get an updated position, and does the same for y.)
     */
    public void updatePosition()
    {
        erase();

        // Adjust the x and y positions based on velocities
        xPosition = xPosition + xVelocity;
        yPosition = yPosition + yVelocity;

        // Draw us in our new position
        draw();
    }

    // ------------ Methods borrowed from Circle from here on down --------------

    /**
     * Make this boulder visible. If it was already visible, do nothing.
     */
    public void makeVisible()
    {
        isVisible = true;
        draw();
    }

    /**
     * Make this boulder invisible. If it was already invisible, do nothing.
     */
    public void makeInvisible()
    {
        erase();
        isVisible = false;
    }

    /**
     * Change the size to the new size (in pixels). Size must be >= 0.
     */
    public void changeSize(int newDiameter)
    {
        erase();
        diameter = newDiameter;
        draw();
    }

    /**
     * Change the color. Valid colors are "red", "yellow", "blue", "green",
     * "magenta" and "black".
     */
    public void changeColor(String newColor)
    {
        color = newColor;
        draw();
    }

    /**
     * Draw the boulder with current specifications on screen.
     */
    private void draw()
    {
        if(isVisible) {
            Canvas canvas = Canvas.getCanvas();
            canvas.draw(this, color, new Ellipse2D.Double(xPosition, yPosition, 
                    diameter, diameter));
        }
    }

    /**
     * Erase the boulder on screen.
     */
    private void erase()
    {
        if(isVisible) {
            Canvas canvas = Canvas.getCanvas();
            canvas.erase(this);
        }
    }

}
